unit Demo1Settings;

interface

uses
{$IFDEF VER120} //  If Delphi 4 then use...
  Windows,
{$ELSE}         //  otherwise use...
  Types,
{$ENDIF}
  PtSettings;

type
  TEmployment = (empNone, empContract, empCasual, empPartTime, empFullTime);
  TEmployments = set of TEmployment;

  TUserSettings = class(TSettingsGroup)
  private
    fAge: integer;
    fName: string;
    fEmployment: TEmployment;
    fEmployments: TEmployments;
  published
    property  Age         : integer       read fAge         write fAge;
    property  Name        : string        read fName        write fName;
    property  Employment  : TEmployment   read fEmployment  write fEmployment;
    property  Employments : TEmployments  read fEmployments write fEmployments;
  end;

  TFormSettings = class(TSettingsGroup)
  private
    fBoundsRect: TRect;
  public
    property  BoundsRect: TRect read fBoundsRect        write fBoundsRect;
  published
    property  Top     : Integer read fBoundsRect.Top    write fBoundsRect.Top;
    property  Left    : Integer read fBoundsRect.Left   write fBoundsRect.Left;
    property  Right   : Integer read fBoundsRect.Right  write fBoundsRect.Right;
    property  Bottom  : Integer read fBoundsRect.Bottom write fBoundsRect.Bottom;
  end;

  TGUISettings = class(TSettingsGroup)
  private
    fMainForm: TFormSettings;
    fOtherForm: TFormSettings;
  public
    constructor Create(const AName: string; const AParent: TSettingsGroup = nil);
    property  MainForm: TFormSettings read fMainForm;
    property  OtherForm: TFormSettings read fOtherForm;
  end;

  TSettings = class(TSettingsGroup)
  private
    fUser: TUserSettings;
    fGUI: TGUISettings;
  public
    constructor Create(const AName: string; const AParent: TSettingsGroup = nil);
    property  User: TUserSettings read fUser;
    property  GUI: TGUISettings read fGUI;
  end;

var
  Settings: TSettings;
  Storage: ISettingsStorage;

implementation

uses
  PtSettingsIni, Demos;

procedure InitializeSettings;
begin
  Settings := TSettings.Create( 'Demo1', nil );
  Storage := TIniSettingsStorage.Create( FileName('.ini') );
  Storage.LoadAll( Settings );
end;

procedure FinalizeSettings;
begin
  Storage.SaveAll( Settings );
  Settings.Free;
  //Storage.Free;   The Storage variable is an interface
end;

{ TGUISettings }

constructor TGUISettings.Create(const AName: string; const AParent: TSettingsGroup);
begin
  inherited;
  fMainForm := TFormSettings.Create('MainForm', Self);
  fOtherForm := TFormSettings.Create('OtherForm', Self);
end;

{ TSettings }

constructor TSettings.Create(const AName: string; const AParent: TSettingsGroup);
begin
  inherited;
  fUser := TUserSettings.Create('User', Self);
  fGUI := TGUISettings.Create('GUI', Self);
end;

initialization
  InitializeSettings;

finalization
  FinalizeSettings;

end.
 
