unit DataList;

interface

uses
  Contnrs;

type
  TDataElem = class
  private
    fLogin: string;
    fXML: string;
  public
    property  Login : string  read fLogin write fLogin;
    property  XML   : string  read fXML   write fXML;
  end;

  TDataList = class
  private
    fData : TObjectList;
    function GetCount: integer;
    function GetElems(i: integer): TDataElem;
  protected
  public
    constructor Create;
    destructor  Destroy; override;
    property  Count : integer read GetCount;
    property  Elems[i:integer]  : TDataElem read GetElems; default;
    procedure Update(const Login: string; const XML: string); //  Doubles as Add() method
    procedure Delete(const Login: string);
    function  IndexOf(const Login: string): integer;
    procedure LoadFromFile(const FileName: string);
    procedure SaveToFile(const FileName: string);
  end;

implementation

uses
  Classes, SysUtils;

{ TDataList }

constructor TDataList.Create;
begin
  inherited;
  fData := TObjectList.Create;
end;

procedure TDataList.Delete(const Login: string);
var
  i : integer;
begin
  i := IndexOf( Login );
  if  i >= 0  then
    fData.Delete( i );
end;

destructor TDataList.Destroy;
begin
  fData.Free;
  inherited;
end;

function TDataList.GetCount: integer;
begin
  Result  := fData.Count
end;

function TDataList.GetElems(i: integer): TDataElem;
begin
  Result  := TDataElem( fData[i] )
end;

function TDataList.IndexOf(const Login: string): integer;
begin
  Result  := Count-1;
  while Result >= 0 do
    if  Elems[Result].Login = Login then
      break
    else
      dec( Result );
end;

procedure TDataList.LoadFromFile(const FileName: string);
var
  Elem : TDataElem;
  fle : Text;
  str1, str2 : string;
begin
  fData.Clear;
  if  not FileExists( FileName )  then
    exit;
  AssignFile( fle, FileName );
  try
    Reset( fle );
    while not Eof( fle )  do
      begin
        Elem  := TDataElem.Create;
        fData.Add( Elem );
        Readln( fle, str1 );
        Readln( fle, str2 );
        Elem.Login  := str1;
        Elem.XML    := str2;
      end;
  finally
    CloseFile( fle )
  end;
end;

procedure TDataList.SaveToFile(const FileName: string);

  function  RemoveNl(Str :string): string;
  var
    p : integer;
  begin
    repeat
      p := Pos( #$A, Str );
      if  p = 0 then
        p := Pos( #$D, Str );
      if  p > 0 then
        Str := Copy( Str, 1, p-1 ) + Copy( Str, p+1, MaxInt );
    until p = 0;
    Result  := Str;
  end;

var
  i : integer;
  fle : Text;
begin
  AssignFile( fle, FileName );
  try
    Rewrite( fle );
    for i := 0 to Count-1 do
      begin
        Writeln( fle, Elems[i].Login );
        Writeln( fle, RemoveNl( Elems[i].XML ) );
      end;
  finally
    CloseFile( fle )
  end;
end;

procedure TDataList.Update(const Login, XML: string);
var
  i : integer;
  Elem : TDataElem;
begin
  i := IndexOf( Login );
  if  i < 0 then
    begin
      Elem  := TDataElem.Create;
      fData.Add( Elem );
      Elem.Login  := Login;
    end
  else
    Elem  := Elems[i];
  Elem.XML  := XML;
end;

end.
 